package org.dive4elements.river.exports.process;

import java.util.Map;

import org.apache.log4j.Logger;
import org.dive4elements.artifactdatabase.state.ArtifactAndFacet;
import org.dive4elements.artifacts.CallContext;
import org.dive4elements.artifacts.CallMeta;
import org.dive4elements.river.artifacts.D4EArtifact;
import org.dive4elements.river.artifacts.access.RiverAccess;
import org.dive4elements.river.artifacts.model.FacetTypes;
import org.dive4elements.river.artifacts.model.WQKms;
import org.dive4elements.river.artifacts.resources.Resources;
import org.dive4elements.river.exports.DiagramGenerator;
import org.dive4elements.river.exports.StyledSeriesBuilder;
import org.dive4elements.river.jfree.StyledXYSeries;
import org.dive4elements.river.themes.ThemeDocument;


public class ComputedDischargeProcessor
extends DefaultProcessor
implements FacetTypes
{
    private static Logger log =
        Logger.getLogger(ComputedDischargeProcessor.class);

    private String yAxisLabel = "";

    private String I18N_AXIS_LABEL = "chart.discharge.curve.yaxis.label";

    @Override
    public void doOut(
            DiagramGenerator generator,
            ArtifactAndFacet bundle,
            ThemeDocument    theme,
            boolean          visible
    ) {
        CallContext context = generator.getCallContext();
        Object data = bundle.getData(context);
        if (data instanceof WQKms) {
            doDischargeOut(generator, bundle, theme, visible);
        }
    }

    @Override
    public String getAxisLabel(DiagramGenerator generator) {
        CallMeta meta = generator.getCallContext().getMeta();
        RiverAccess access = new RiverAccess((D4EArtifact)generator.getMaster());
        String unit = access.getRiver().getWstUnit().getName();
        if (yAxisLabel != null && !yAxisLabel.isEmpty()) {
            return Resources.getMsg(
                meta,
                yAxisLabel,
                new Object[] { unit });
        }
        return Resources.getMsg(
                meta,
                I18N_AXIS_LABEL,
                new Object[] { unit });
    }

    /**
     * Returns true if the Processor class is able to generate output for a facet type
     *
     * @param facettype Name of the facet type
     * @return true if the facettype can be processed
     */
    @Override
    public boolean canHandle(String facettype) {
        return COMPUTED_DISCHARGE_Q.equals(facettype);
    }

    private void doDischargeOut(
        DiagramGenerator generator,
        ArtifactAndFacet bundle,
        ThemeDocument theme,
        boolean visible
    ) {
        CallContext context = generator.getCallContext();

        Object data = bundle.getData(context);
        WQKms wqkms = (WQKms) data;

        Object currentKm = context.getContextValue("currentkm");
        if (currentKm instanceof Number) {
            double ckm = ((Number)currentKm).doubleValue();
            Double skm = wqkms.sameKm();
            if (skm != null && Math.abs(skm-ckm) > 0.00001) {
                return;
            }
        }

        Map<String, String> metaData = bundle.getFacet().getMetaData(
            bundle.getArtifact(), context);

        StyledXYSeries series =
            new StyledXYSeries(bundle.getFacetDescription(), theme);

        series.putMetaData(metaData, bundle.getArtifact(), context);
        yAxisLabel = metaData.get("Y");



        double[][] wqData = new double[2][wqkms.size()];
        for (int i = 0; i < wqkms.size(); i++) {
            wqData[0][i] = wqkms.getQ(i);
            wqData[1][i] = wqkms.getW(i);
        }
        StyledSeriesBuilder.addPoints(series, wqData, false);
        generator.addAxisSeries(series, axisName, visible);
    }
}
